@@ -129,8 +129,8 @@ Return the result as a JSON object in the following format:
     },
     "questions": "Questions and suggestions for the authors",
     "limitations": "Limitations of the paper",
-    "overall": ...,
-    "confidence": ...,
+    "overall": "Number + short description",
+    "confidence": "Number + short description",
     "format_issues": "Format issues",
 }
 """
