@@ -17,7 +17,8 @@ def web_search(
     provider: Optional[str] = "tavily",
 ) -> str:
     """
-    Search the web using Exa Search or Tavily and return normalized results.
+    Search the web using Exa Search, Brave Search or Tavily and return normalized results.
+    If the specified provider is not available, the function will try to use the next available provider.
 
     Returns a JSON object serialized to a string. The structure is: {"results": [...]}
     Every item in the "results" has at least the following fields: ("title", "url")
@@ -28,17 +29,34 @@ def web_search(
         limit: The maximum number of items to return. 20 by default, maximum 25.
         provider: The provider to use. "exa", "tavily" or "brave". "tavily" by default.
     """
-    assert provider in (
-        "exa",
-        "tavily",
-        "brave",
-    ), "Error: provider must be either 'exa', 'tavily' or 'brave'"
+    providers = ("tavily", "brave", "exa")
+    assert provider in providers, "Error: provider must be either 'exa', 'tavily' or 'brave'"
 
+    is_tavily_available = os.getenv("TAVILY_API_KEY") is not None
+    is_exa_available = os.getenv("EXA_API_KEY") is not None
+    is_brave_available = os.getenv("BRAVE_API_KEY") is not None
+    assert is_tavily_available or is_exa_available or is_brave_available
+    availability = {
+        "tavily": is_tavily_available,
+        "brave": is_brave_available,
+        "exa": is_exa_available,
+    }
+
+    if not availability[provider]:
+        for p in providers:
+            if availability[p]:
+                provider = p
+                break
+
+    result = {}
     if provider == "exa":
-        return exa_web_search(query, limit)
-    if provider == "brave":
-        return brave_web_search(query, limit)
-    return tavily_web_search(query, limit)
+        result = json.loads(exa_web_search(query, limit))
+    elif provider == "brave":
+        result = json.loads(brave_web_search(query, limit))
+    elif provider == "tavily":
+        result = json.loads(tavily_web_search(query, limit))
+    result["search_provider"] = provider
+    return json.dumps(result, ensure_ascii=False)
 
 
 def tavily_web_search(query: str, limit: Optional[int] = 20) -> str:
@@ -68,6 +86,12 @@ def tavily_web_search(query: str, limit: Optional[int] = 20) -> str:
     }
     response = post_with_retries(TAVILY_SEARCH_URL, payload, key)
     results = response.json()["results"]
+    for result in results:
+        content = " ".join(result["content"].split(" ")[:40])
+        content = content.strip("., ")
+        result["content"] = content
+        result.pop("raw_content", None)
+        result.pop("score", None)
     return json.dumps({"results": results}, ensure_ascii=False)
 
 
