@@ -13,3 +13,8 @@ def test_visit_webpage_exa() -> None:
     assert content is not None
     assert "Example Domain" in content
     assert "illustrative" in content
+
+
+def test_visit_webpage_pdf() -> None:
+    content = visit_webpage("https://arxiv.org/pdf/2409.06820")
+    assert "A Benchmark for Role-Playing" in content
